package Domain;
import java.util.*;

public class Neuron{

    private double m_output;//[0,1]
    private double m_sum;//weighted sum
    private double m_delta;
    private Vector m_inputLinks;//input synapses
    private Vector m_outputLinks;//output synapse
    private String m_label;

  public Neuron(String l){

    m_output        = 0.0;
    m_delta         = 0.0;
    m_sum           = 0.0;
    m_inputLinks    = new Vector();
    m_outputLinks   = new Vector();
    m_label         = new String(l);
  }
  //*****************     Getter And Setter        *******************
  //***************************************************************
  public double getOutput(){
    return m_output;
  }

  public void setOutput(double m_output) {
        this.m_output = m_output;
  }

    public double getDelta(){
    return m_delta;
  }

  public void addInputSynapse(Synapse s){
      m_inputLinks.addElement(s);
  }

  public void addOutputSynapse(Synapse s){
      m_outputLinks.addElement(s);
  }



  //*******************   BACKPROPAGATION ALGORITHM   ********
  //**********************************************************
  // computeOutput
  // computeBackpropDelta for output Neuron
  // computeBackpropDelta for hidden layer Neuron
  // computeWeight
  public void computeOutput(){
    Enumeration e = m_inputLinks.elements();
    Synapse s;
    m_sum=0.0;
    while(e.hasMoreElements()){
      s = (Synapse)e.nextElement();
      m_sum += s.getFromNeuron().getOutput()*s.getWeight();
    }
    m_output = Library.Definitions.sigmoid(m_sum);
  }

  public void computeBackpropDelta(double target){ // for an output neuron
    m_delta = (target - m_output) * m_output * (1.0 - m_output);
  }

  public void computeBackpropDelta(){ // for a hidden neuron
    double sumOfError = 0.0;
    Synapse synapse;
    Enumeration e = m_outputLinks.elements();
    while(e.hasMoreElements()){
      synapse = (Synapse)e.nextElement();
      sumOfError += synapse.getToNeuron().getDelta() * synapse.getWeight();
    }
    m_delta = sumOfError * m_output * ( 1.0 - m_output);
  }
  public void computeWeight(){
    Synapse synapse;
    Enumeration e = m_inputLinks.elements();
    while(e.hasMoreElements()){
      synapse = (Synapse)e.nextElement();
      double deltaWeight = Library.Definitions.LEARNING_RATE *
                            m_delta *
                            synapse.getFromNeuron().getOutput();
      double newWeight = synapse.getWeight()+deltaWeight;
      synapse.setData(deltaWeight);
      synapse.setWeight(newWeight);
    }
  }
  public void printNeuron(){

    System.out.print(m_label+"="+m_output+": ");
    Synapse synapse;
    Enumeration e = m_outputLinks.elements();
    while(e.hasMoreElements())
    {
      synapse = (Synapse)e.nextElement();
      System.out.print(synapse.getToNeuron().m_label+"("+synapse.getWeight()+") ");
    }
    System.out.println("");
  }
}
