package Domain;

import java.util.Vector;
import java.util.Enumeration;


public class Perceptron{
    private  Vector  m_layers;
    private  Vector  m_inputSamples;
    private  Vector  m_outputSamples;
    private  Layer   m_inputLayer;
    private  Layer   m_outputLayer;
    private  double  m_error;

 public Perceptron(int i,int o){
    m_layers          = new Vector();
    m_inputSamples    = new Vector();
    m_outputSamples   = new Vector();
    m_inputLayer      = new Layer("I",i+1); // plus the bias
    m_outputLayer     = new Layer("O",o);
    m_layers.addElement(m_inputLayer);
    m_layers.addElement(m_outputLayer);
    m_error = 0.0;
 }

 public void addLayer(int n,String name){
    m_layers.insertElementAt(new Layer(name,n),m_layers.size()-1);
 }

 public Layer getLayer(int i){
    int         j=0;
    boolean     found=false;
    Layer       layer=null;
    Enumeration e = m_layers.elements();
    while(e.hasMoreElements()){
        layer = (Layer)e.nextElement();
        if (i==j){
            found = true;
            break;
        }
        else j++;
    }
    if (found==false)
        layer = null;
    return layer;
 }

 public void connect(int sourceLayer,int sourceNeuron,
                          int destLayer,int destNeuron){
    new Synapse(getLayer(sourceLayer).getNeuron(sourceNeuron),
                getLayer(destLayer).getNeuron(destNeuron));
 }

 public void biasConnect(int destLayer,int destNeuron){
    new Synapse(m_inputLayer.getNeuron(m_inputLayer.getNumOfNeurons()-1),
                    getLayer(destLayer).getNeuron(destNeuron));
 }

 public void removeSamples(){
    m_inputSamples.removeAllElements();
    m_outputSamples.removeAllElements();
 }

 public void addSample(Vector i,Vector o){
    m_inputSamples.addElement(i);
    m_outputSamples.addElement(o);
 }

 public void setSamples(Vector i, Vector o){
     m_inputSamples     = i;
     m_outputSamples    = o;
 }

 public void printSamples(){
        System.out.println(m_inputSamples+"->"+m_outputSamples);
 }

 //             Recognize after Learning
 //             -------------------------
 public Vector recognize(Vector iS){
    initInputs(iS);
    propagate();
    Vector oS = getOutput();
    return oS;
 }
 //             Learn From Data Set
 //           ----------------------------
 public void learn(int iterations){
    Enumeration iS;
    Enumeration oS;
    for(int i = 0; i<iterations; i++){
        // accumulate total error over each epoch
        m_error = 0.0;
        iS = m_inputSamples.elements();
        oS = m_outputSamples.elements();
        while(iS.hasMoreElements()){
            Vector temp = (Vector) oS.nextElement ();
            learnPattern((Vector)iS.nextElement(),temp);
            m_error += computeError (temp);
        }
    }
 }
  void learnPattern(Vector iS, Vector oS){
    initInputs(iS);
    propagate();
    bpAdjustWeights(oS);
  }

  void initInputs(Vector iS){
    Neuron neuron;
    Enumeration eInputLayer = m_inputLayer.getNeurons().elements();
    Enumeration eSample = iS.elements();
    while (eSample.hasMoreElements()){
        neuron = (Neuron)eInputLayer.nextElement();
        neuron.setOutput(((Double)eSample.nextElement()).doubleValue());
    }
    neuron = (Neuron)eInputLayer.nextElement(); // bias;
    neuron.setOutput(1.0);
  }

  void propagate(){
    Layer layer;
    Enumeration e = m_layers.elements();
    e.nextElement(); // skip the input layer
    while(e.hasMoreElements()){
        layer = (Layer)e.nextElement();
        layer.computeOutputs();
    }
 }

  void bpAdjustWeights(Vector oS){
    m_outputLayer.computeBackpropDeltas(oS);
    for(int i=m_layers.size()-2; i>=1; i--){
        ((Layer)m_layers.elementAt(i)).computeBackpropDeltas();
    }
    m_outputLayer.computeWeights();
    for(int i=m_layers.size()-2; i>=1; i--)
         ((Layer)m_layers.elementAt(i)).computeWeights();
  }

  //Return output As Vector of Double
  public Vector getOutput(){
    Vector oS = new Vector();
    Neuron neuron;
    Enumeration e = m_outputLayer.getNeurons().elements();
    while(e.hasMoreElements()){
        neuron = (Neuron) e.nextElement();
        oS.addElement(new Double(neuron.getOutput()));
    }
    return oS;
 }
 //Calc Error
 double computeError(Vector oS){
     Neuron neuron;
     double sum=0.0;
     double tmp;
     Enumeration e = m_outputLayer.getNeurons().elements();
     Enumeration eOS = oS.elements();
     while (e.hasMoreElements()){
        neuron = (Neuron)e.nextElement();
        tmp = ((Double)eOS.nextElement()).doubleValue() - neuron.getOutput();
        sum += tmp * tmp;
     }
        return sum/2.0;
 }

 double currentError() {
    return m_error;
 }

 void printPerceptron(){
    Layer layer;
    Enumeration e = m_layers.elements();
    while(e.hasMoreElements()){
        layer = (Layer)e.nextElement();
        layer.printLayer();
    }
 }
}

